#include "test.c"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>

#include "atax_link.tiled.c"


int extract_int_arg(char** argv, int num) {
  char* val = argv[num];
  int res = atoi(val);
  return res;
}



int main(int argc, char** argv) {
  //Check number of args
  if (argc != 3) {
    printf("Expecting 2 arguments, not %i.\n", (argc-1));
    printf("Expecting M, N\n");
    exit(0);
  }

  int M = extract_int_arg(argv, 1);
  int N = extract_int_arg(argv, 2);
  
  
  //
  // Tiled kernel
  //

  //Init inputs
  POLYBENCH_1D_ARRAY_DECL(x,double,N,N);
  POLYBENCH_2D_ARRAY_DECL(A,double,M,N, M,N);
  POLYBENCH_1D_ARRAY_DECL(y,double,N,N);
  POLYBENCH_1D_ARRAY_DECL(tmp,double,M,M);
  
  init1_2(N, N, &elem_polyb_1_4, POLYBENCH_ARRAY(x));
  init2_4(M,N, N, M, &elem_polyb_2_2_3, POLYBENCH_ARRAY(A));
  init1_1(N,&elem_polyb_1_2, POLYBENCH_ARRAY(y));
  init1_1(M,&elem_polyb_1_2, POLYBENCH_ARRAY(tmp));

  
  /* Start timer. */
  polybench_start_instruments;
  
  //Call the main computation
  tiled_kernel(M, N, POLYBENCH_ARRAY(x), POLYBENCH_ARRAY(A), POLYBENCH_ARRAY(y), POLYBENCH_ARRAY(tmp));

  /* Stop and print timer. */
  polybench_stop_instruments;
  polybench_print_instruments;

  /* Prevent dead-code elimination. All live-out data must be printed
     by the function call in argument. */
  polybench_prevent_dce(print1(N, POLYBENCH_ARRAY(y)));

  /* Be clean. */
  POLYBENCH_FREE_ARRAY(x);
  POLYBENCH_FREE_ARRAY(A);
  POLYBENCH_FREE_ARRAY(y);
  POLYBENCH_FREE_ARRAY(tmp);

  return 0;
}
