#include "test.c"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>

#include "jacobi-1d_link.tiled.c"


int extract_int_arg(char** argv, int num) {
  char* val = argv[num];
  int res = atoi(val);
  return res;
}



int main(int argc, char** argv) {
  //Check number of args
  if (argc != 4) {
    printf("Expecting 3 arguments, not %i.\n", (argc-1));
    printf("Expecting N, TSTEPS, b\n");
    exit(0);
  }

  int N = extract_int_arg(argv, 1);
  int TSTEPS = extract_int_arg(argv, 2);
  int block_size = extract_int_arg(argv, 3); 

  int outer_N = N / block_size;
  int inner_N = N % block_size;
  int outer_TSTEPS = TSTEPS / block_size;
  int inner_TSTEPS = TSTEPS % block_size;
  //
  // Tiled kernel
  //

  //Init inputs
  POLYBENCH_1D_ARRAY_DECL(A,double,N,N);
  POLYBENCH_1D_ARRAY_DECL(B,double,N,N);
  
  init1_2(N, N, &elem_polyb_1_6, POLYBENCH_ARRAY(A));
  init1_2(N, N, &elem_polyb_1_7, POLYBENCH_ARRAY(B));

  /* Start timer. */
  polybench_start_instruments;

  //Call the main computation
  tiled_kernel(N, TSTEPS, outer_N, inner_N, outer_TSTEPS, inner_TSTEPS,
    POLYBENCH_ARRAY(A), POLYBENCH_ARRAY(B), block_size);

//  printf("PING !!!\n");

  /* Stop and print timer. */
  polybench_stop_instruments;

//  printf("PANG !!!\n");


  polybench_print_instruments;

//  printf("PONG !!!\n");

  /* Prevent dead-code elimination. All live-out data must be printed
     by the function call in argument. */
  polybench_prevent_dce(print1(N, POLYBENCH_ARRAY(A)));

  /* Be clean. */
  POLYBENCH_FREE_ARRAY(A);
  POLYBENCH_FREE_ARRAY(B);


  return 0;
}
